/*
 * NVS Sample for Zephyr using high level API, the sample illustrates the usage
 * of NVS for storing data of different kind (strings, binary blobs, unsigned
 * 32 bit integer) and also how to read them back from flash. The reading of
 * data is illustrated for both a basic read (latest added value) as well as
 * reading back the history of data (previously added values). Next to reading
 * and writing data it also shows how data can be deleted from flash.
 *
 * The sample stores the following items:
 * 1. A string representing an IP-address: stored at id=1, data="192.168.1.1"
 * 2. A binary blob representing a key: stored at id=2, data=FF FE FD FC FB FA
 *    F9 F8
 * 3. A reboot counter (32bit): stored at id=3, data=reboot_counter
 * 4. A string: stored at id=4, data="DATA" (used to illustrate deletion of
 * items)
 *
 * At first boot the sample checks if the data is available in flash and adds
 * the items if they are not in flash.
 *
 * Every reboot increases the values of the reboot_counter and updates it in
 * flash.
 *
 * At the 10th reboot the string item with id=4 is deleted (or marked for
 * deletion).
 *
 * At the 11th reboot the string item with id=4 can no longer be read with the
 * basic nvs_read() function as it has been deleted. It is possible to read the
 * value with nvs_read_hist()
 *
 * At the 78th reboot the first sector is full and a new sector is taken into
 * use. The data with id=1, id=2 and id=3 is copied to the new sector. As a
 * result of this the history of the reboot_counter will be removed but the
 * latest values of address, key and reboot_counter is kept.
 *
 * Copyright (c) 2018 Laczen
 *
 * SPDX-License-Identifier: Apache-2.0
 */
#define MODULE main



#include <zephyr/kernel.h>
#include <zephyr/sys/reboot.h>
#include <zephyr/device.h>
#include <string.h>
#include <zephyr/drivers/flash.h>
#include <zephyr/storage/flash_map.h>
#include <zephyr/fs/nvs.h>

#include <cjson.h>

#include <zephyr/logging/log.h>
LOG_MODULE_REGISTER(MODULE, LOG_LEVEL_DBG);

static struct nvs_fs fs;

#define NVS_PARTITION			storage_partition
#define NVS_PARTITION_DEVICE	FIXED_PARTITION_DEVICE(NVS_PARTITION)
#define NVS_PARTITION_OFFSET	FIXED_PARTITION_OFFSET(NVS_PARTITION)

#define CONFIG_DATA_ID 1
#define DEFAULT_CONFIG_STR "{\"a\":1, \"b\":\"string\", \"c\":[1,2,3]}"

static char default_conf[4096];
static char my_buf[4096];

static cJSON* config;

static struct flash_pages_info info;

int setup_flash() 
{
	int rc = 0;
	fs.flash_device = NVS_PARTITION_DEVICE;
	if (!device_is_ready(fs.flash_device)) {
		LOG_INF("Flash device %s is not ready", fs.flash_device->name);
		return -1;
	}
	fs.offset = NVS_PARTITION_OFFSET;
	rc = flash_get_page_info_by_offs(fs.flash_device, fs.offset, &info);
	if (rc) {
		LOG_ERR("Unable to get page info");
		return -1;
	}
	fs.sector_size = info.size;
	fs.sector_count = 4U;

	rc = nvs_mount(&fs);
	if (rc) {
		LOG_INF("Flash Init failed");
		return -1;
	}
	return rc;
}

static void read_config() {
	strcpy(default_conf, DEFAULT_CONFIG_STR);
	size_t load_amt = strlen(default_conf);

	int rc = nvs_read(&fs, CONFIG_DATA_ID, &my_buf, load_amt+1);
	if (rc > 0) {
		/* item was found, show it */
		LOG_INF("loaded %d chars from ID[%d]", load_amt, CONFIG_DATA_ID);
		LOG_INF("Data Loaded: %s", my_buf);
		config = cJSON_Parse(my_buf);
		LOG_INF("Data Parsed: \n%s", cJSON_Print(config));
	}
	else { // if not found
		LOG_WRN("Error! Cannot read config (%d)", rc);
	}	

	// Write new content
	LOG_INF("Writing default_conf, len = %d", strlen(default_conf));
	rc = nvs_write(&fs, CONFIG_DATA_ID, &default_conf, load_amt+1 );
	if(rc >= 0) {
		LOG_INF("Wrote %d chars to nvs", rc);
	}
	else {
		LOG_ERR("Failed to write to NVS (%d)", rc);
	}
}

int main(void)
{
	k_msleep(2000);
	int rc = setup_flash();
	if(rc) {
		LOG_ERR("Setup Flash Failed (%d)", rc);
	}
	k_msleep(100);
	read_config();
	return 0;
}
